/*
 *   read Sagittal decimal<TAB>numerator<TAB>denominator<EOL>
 *
 *   Generate macros to convert from sagittal to abc2midi microtones
 *
 *   TODO: treat temperings
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

static double interv2cents(double interv);
static double cents2interv(double cents);
static int to_microtone(int dec, double num, double den);
static double factor(double x, double y);
static double tempering(double cents, double powfifth);
static double str2ratio(char *s);

#define MAX(x,y) ((x)>(y)?(x):(y))

int sr = 4096; /* semitone fraction for microtones (pitch bend) */

/* reference intervals */
double temperfifth = 1.5;
double temperoctave = 2.0;
#define OCTAVE 2.0
#define FIFTH  1.5
#define FIFTH3 3.0

static double sagittal[256][3];
static int nsag=0;

int main(int argc, char **argv)
{
    char buf[BUFSIZ];
    int dec, sr, status, i;
    int out;
    double num,den;
    double x,y,m,n;

    nsag=0;
    status=0;

    if(argc>=2) {
	temperfifth = str2ratio(argv[1]);
	printf("# fifthsize=%lf (%lf cents)\n",
	       temperfifth,interv2cents(temperfifth));
    }

    while(status<2) { /* read input */
	status+=(fgets(buf,BUFSIZ,stdin)==NULL);
	//printf("status=%d; [%s] ",status,buf);

	if(status) { /* FIXME: treat blank lines at end of file */
	    dec=num=den=0;
	    break;
	}

	sscanf(buf,"%d %lf %lf",&dec,&num,&den);
	if(dec && num && den) {
	    //printf("dec=%d num=%.f den=%.f\n",dec,num,den);
	    sagittal[nsag][0]=(double)dec;
	    sagittal[nsag][1]=num;
	    sagittal[nsag][2]=den;
	    if(nsag<=255)
		nsag++;
	}
	num=0; den=1;
    }
    
    for(i=0;i<nsag;i++) { /* generate output */
	dec = (int) (sagittal[i][0]);
	//	fprintf(stderr,"dec=%d ",dec);
	num = sagittal[i][1];
	den = sagittal[i][2];
	to_microtone(dec,num,den);
    }
    
}


static double interv2cents(double interv)
{
    return 1200.0 * (log(interv)/log(2.0));
}

static double cents2interv(double cents)
{
    return pow(2.0,cents/1200.0);
}

static int to_microtone(int dec, double num, double den)
{

    double cents, powf;
    powf=0.0;
    cents=interv2cents(num/den);

    if(temperfifth != FIFTH) { /* tempering */
	powf = factor(num,den);
	// printf("num:%lf den:%lf powf: %lf\n",num,den,powf);
	if(powf==0.0) {
	    fprintf(stderr,
		    "\aUnchanged when tempering: "
		    "char=%3d   num=%.lf den=%.lf  \tcents=%lf, %.lf/%.lf\n",
		    dec, num,den, cents,cents*sr/100.0, (double)sr);
	}

	cents = tempering(cents,powf);
    }

    if(cents==0.0) { /* natural */
	printf("#define ");
	printf("^/%d =\n",dec);
	printf("#define ");
	printf("_/%d =\n",dec);
    }
    else if(cents>0.0) { /* sharps */
	printf("#define ");
	printf("^/%d ^%d/%d\n",dec,(int)(cents*sr)/100,sr);
	printf("#define ");
	printf("_/%d ^%d/%d\n",dec,(int)(cents*sr)/100,sr);
    }
    else { /* flats */
	cents=-cents;
	printf("#define ");
	printf("^/%d _%d/%d\n",dec,(int)(cents*sr)/100,sr);
	printf("#define ");
	printf("_/%d _%d/%d\n",dec,(int)(cents*sr)/100,sr);
    }
}

static double factor(double x, double y)
{
    double powe, div;

    powe=0.0;
    div=x;
    while( div==floor(div) ) {
	div=div/FIFTH3;
	powe++;
    }
    div=y;
    while( div==floor(div) ) {
	div=div/FIFTH3;
	powe--;
    }

    //printf("powe: %lf\n",powe);
    return powe;
}

/* temper the fifth */
static double tempering(double cents, double powfifth)
{
    double x,y;

    x=interv2cents(FIFTH/temperfifth);
    y=(powfifth * x);
    cents-=y;
    //    printf("powfifth=%lf, x=%lf, y=%lf, cents=%lf\n",powfifth,x,y,cents);
    return cents;
    /* TODO: temper the octave */
}

static double str2ratio(char *s)
{
    double n,d;
    int c;
    if((c=sscanf(s,"%lf/%lf\n",&n,&d))>0) {
	if(c==1 && strchr(s,'.')!=NULL)
	    return cents2interv(n);
	else if(d!=0.0)
	    return (n/d);
    }
    return 0.0;
}
