/* pure2mixed - Hudson Lacerda (2006) */
/* Generate postscript code to convert from pure to mixed-symbol form
   (Sagittal-2.0) */
/* (According to instructions from George Secor) */

#include <stdio.h>

int pure2mixed(unsigned int d, unsigned int *m, unsigned int *n);
int out_ps(unsigned int d, unsigned int m, unsigned n);

#define DFLAT ((unsigned) 33)
#define FLAT ((unsigned) 35)
#define SHARP ((unsigned) 253)
#define DSHARP ((unsigned) 255)
#define NATURAL ((unsigned) 144)

#define MAINTAIN (0)  /* flag for no conversion */
#define SINGLE (0)    /* flag for single symbol */

#define SAG_DFLAT (40)
#define SAG_DSHARP (248)

int main()
{
    unsigned int i,m,n;
    for(i=SAG_DFLAT; i<=SAG_DSHARP; i++) {
	pure2mixed(i,&m,&n);
	out_ps(i,m,n);
    }
}

int pure2mixed(unsigned int d,unsigned int *m, unsigned int *n)
{

    *m=*n=0;

    /*
      Yes, there are 11 different cases, depending on the ASCII decimal value
      of the pure symbol:
    */

    /*
      Replace a Sagittal double-flat symbol (decimal value 40) with a
      conventional double-flat (decimal value 33).
    */

    if(d==40)
	*m=DFLAT, *n=SINGLE;

    /*
      For the other crossed-shaft down-symbols (decimal values 41 thru 60),
      add 104 to its ASCII decimal value (to convert to the single-shaft
      opposite direction apotome-complement symbol) and put a conventional
      double-flat symbol (33) to its right.
    */
    
    else if(d>=41 && d<=60)
	*m=d+104, *n=DFLAT;

    /*
      For the triple-shaft down-symbols (61 thru 91), add 52 to the ASCII
      decimal value (to convert to single-shaft) and put a conventional flat
      symbol (decimal value 35) to its right.
    */

    else if(d>=61 && d<=91)
	*m=d+52, *n=FLAT;

    /*
      Replace a Sagittal flat symbol (92) with a conventional flat (35).
    */

    else if(d==92)
	*m=FLAT, *n=SINGLE;

    /*
      For the double-shaft down-symbols (93 thru 112), add 52 to its ASCII
      decimal value (to convert to the single-shaft opposite direction
      apotome-complement symbol) and put a conventional flat symbol (35) to
      its right.
    */
    
    else if(d>=93 && d<=112)
	*m=d+52, *n=FLAT;

    /*
      For the single-shaft symbols (113 thru 175, including the natural sign)
      the pure and mixed notation is the same, so no conversion is necessary.
    */

    else if(d>=113 && d<=175)
	*m=MAINTAIN; /* don't convert! */

    /*
      For the double-shaft up-symbols (176 thru 195), subtract 52 from its
      ASCII decimal value (to convert to the single-shaft opposite direction
      apotome-complement symbol) and put a conventional sharp symbol (decimal
      value 253) to its right.
    */

    else if(d>=176 && d<=195)
	*m=d-52, *n=SHARP;

    /*
      Replace a Sagittal sharp symbol (196) with a conventional sharp (253).
    */

    else if(d==196)
	*m=SHARP, *n=SINGLE;

    /*
      For the triple-shaft up-symbols (197 thru 227), subtract 52 from the
      ASCII decimal value (to convert to single-shaft) and put a conventional
      sharp symbol (decimal value 253) to its right.
    */

    else if(d>=197 && d<=227)
	*m=d-52, *n=SHARP;

    /*
      For the crossed-shaft up-symbols (228 thru 247), subtract 104 from its
      ASCII decimal value (to convert to the single-shaft opposite direction
      apotome-complement symbol) and put a conventional sharp symbol (decimal
      value 253) to its right.
    */
    
    else if(d>=228 && d<=247)
	*m=d-104, *n=DSHARP; /* fixed: double sharp */
    /* *m=d-104, *n=SHARP; */

    /*
      Replace a Sagittal double-sharp symbol (248) with a conventional
      double-sharp (255).

      --George
    */

    else if(d==248)
	*m=DSHARP, *n=SINGLE;

    else {
	*m=MAINTAIN;
	fprintf(stderr,"Unknown accidental code %d\n",d);
    }
}

int out_ps(unsigned int d, unsigned int m, unsigned n)
{
    if(m==MAINTAIN)
	return 0;
    if(n==SINGLE) {
	if(d<NATURAL)
	    printf("postscript /ft%d/ft%d load bind def\n",d-1,m-1);
	else
	    printf("postscript /sh%d/sh%d load bind def\n",d-1,m-1);
	return 0;
    }
    printf("postscript /");
    if(d<NATURAL)
	printf("ft");
    else
	printf("sh");
    printf("%d{(\\%o\\%o)sags}!\n",d-1,m,n);
    return 0;
}

