/* Module to compute/convert musical intervals/pitches/frequencies */
/* This file is part of microabc (C) 2006-2008 Hudson Lacerda */

/*
 * Copyright (C) 2006-2008 Hudson Flvio Meneses Lacerda
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* SUJEITO  LICENA PBLICA GERAL DO GNU (GNU GPL)
 * TEXTO DA LICENA EM:
 *	ARQUIVO "Licenca.txt"
 *      <http://www.gnu.org/copyleft/gpl.html>
 *      <http://www.magnux.org/doc/GPL-pt_BR.txt>
 */

#include "interval.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

int diatonic(int p)
{
    int s[7]={0,2,4,5,7,9,11}, i;
    for(i=0;i<7;i++)
	if(mod(p,12)==s[i])
	    return 1;
    return 0;
}

int mod(int n, int m)
{
    if(m<2)
	return 0;
    n= n%m;
    while(n<0)
	n+=m;
    return n;
}

double f_mod(double f, double m)
{
    double x;
    // double trunc(double x); /* math.h */
    x = f/m;
    x = (x-trunc(x))*m;
    while(x<0)
	x+=m;
    while(x>m)
	x-=m;
    return x;
}

int equaltemp(int n, double m /*ratio*/, double scale[])
{
    int i;
    for(i=0; i<n; i++)
	scale[i]=interv2cents(pow(m,(double)i/(double)n));
    return n;
}

int equaltempc(int n, double m /*cents*/, double scale[])
{
    int i;
    m/=n;
    for(i=0; i<n; i++)
	scale[i]=i*m;
    return n;
}

int lineartemp(int n,double m,double generator,
		   int countdownwards,
		   double scale[])
{
    int i;

    if(n==0)
	return 0;

    m = interv2cents(m);
    m = (m >= 0) ? m : -m;

    generator = interv2cents(generator);
    /* TODO: improve; this is the simpler way, maybe slow */
    while(generator<0.0) /* put inside modulo */
	generator+=m;
    while(generator>m)
	generator-=m;

    for(i=0;i<=n;i++) {
	scale[i] = (i-countdownwards)*generator;
	/* TODO: improve; this is the simpler way, maybe slow */
	while(scale[i]<0.0) /* put inside modulo */
	    scale[i]+=m;
	while(scale[i]>m)
	    scale[i]-=m;
    }

    qsort(scale, (size_t)n, (size_t)sizeof(scale[0]),
	  (void *) fcompare );
	/*
 void qsort(void *base, size_t nmemb, size_t size,
                  int(*compar)(const void *, const void *));
	*/
    return n;
}

int lineartempc(int n,double m /*cents*/,double generator /*cents*/,
		   int countdownwards,
		   double scale[])
{
    int i;

    if(n==0)
	return 0;

    m = (m >= 0) ? m : -m;

    /* TODO: improve; this is the simpler way, maybe slow */
    while(generator<0.0) /* put inside modulo */
	generator+=m;
    while(generator>m)
	generator-=m;

    for(i=0;i<=n;i++) {
	scale[i] = (i-countdownwards)*generator;
	/* TODO: improve; this is the simpler way, maybe slow */
	while(scale[i]<0.0) /* put inside modulo */
	    scale[i]+=m;
	while(scale[i]>m)
	    scale[i]-=m;
    }

    qsort(scale, (size_t)n, (size_t)sizeof(scale[0]),
	  (void *) fcompare );
	/*
 void qsort(void *base, size_t nmemb, size_t size,
                  int(*compar)(const void *, const void *));
	*/
    return n;
}

int fcompare(const double *x, const double *y)
{
    if (*x == *y)
	return 0;
    return (*x > *y) ? 1 : -1;
}

double interv2cents(double interv)
{
    /* TODO: Is there a yet more precise way with double? */
    /* long double log2l(long double x); /// math.h */

    return (double) ((long double) 1200.0 * log2l((long double) interv));

    /* return 1200.0 * (log(interv) / log(2.0)); */

}

double cents2interv(double cents)
{
    /* TODO: Is there a more precise way with double? */
    /* long double exp2l(long double x); /// math.h */
    return (double) exp2l((long double) cents / (long double) 1200.0);
    /* return exp(log(2.0) * cents / 1200.0); */
    /* return pow(2.0,cents/1200.0); */
}

/* convert from cents to semitone fraction with given denominator */
int cents2frac(double cents, int den)
{
    /* TODO: check roundup */
    /* return (int) floor ((cents/100.0) * den + .5); */
    return (int) round ((cents/100.0) * den);
}

double freq2midi(double freq, double a_freq)
{
    /*  long double exp2l(long double x); // math.h */
    /* long double log2l(long double x); // math.h */
    long double f0;

    /*f0 = a_freq * pow(2.0, (-69.0/12.0));*/ /* freq for MIDI C0 */

    /* freq for MIDI C0 */
    f0 = (long double)
	a_freq * exp2l(((long double)-69.0/(long double)12.0));

    /*return 12.0 * log(freq/f0) / log(2.0);*/
    return (double)
	((long double)12.0 * log2l((long double)freq/f0));
}

double midi2freq(double midi, double a_freq)
{
    /* long double exp2l(long double x); // math.h */

    /*return a_freq * pow(2.0, (midi-69.0)/12.0);*/
    return (double)
	((long double)
	a_freq * exp2l((long double)(midi-69.0)/(long double)12.0));
}

int nearestdiat(double pitch)
{
    int temp1, temp2;
    temp1= (int) floor(pitch+.5);
    temp2= (int) ceil(pitch-.5);

    if((round(pitch)==pitch) && diatonic((int)pitch))
	return (int) pitch;
    while(!diatonic(temp1))
	temp1--;
    while(!diatonic(temp2))
	temp2++;

    if(abs(temp1-pitch) <= abs(temp2-pitch))
	return temp1;
    return temp2;
}

