/* Module to compute/convert musical intervals/pitches/frequencies */
/* This file is part of microabc (C) 2006-2008 Hudson Lacerda */

/*
 * Copyright (C) 2006-2008 Hudson Flvio Meneses Lacerda
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* SUJEITO  LICENA PBLICA GERAL DO GNU (GNU GPL)
 * TEXTO DA LICENA EM:
 *	ARQUIVO "Licenca.txt"
 *      <http://www.gnu.org/copyleft/gpl.html>
 *      <http://www.magnux.org/doc/GPL-pt_BR.txt>
 */

#ifndef _INTERVAL__H_
#define _INTERVAL__H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


/* math.h */
#ifdef CYGWIN /* long-double functions disabled for CYGWIN */
#ifndef powl
#define powl(x,y) pow((double)(x),(double)(y))
#endif
#ifndef log2l
#define log2l(x) (log((double)(x)) / log(2.0))
#endif
#ifndef exp2l
#define exp2l(x) pow(2.0,(double)(x))
#endif
#ifndef trunc
#define trunc(x) ((double)((long int)(x)))
#endif
#ifndef round
#define round(x) ((double)((long int) ((x) +  ((x)>0?.5:(-.5))  )))
#endif
#else /* not CYGWIN */
long double powl(long double x, long double y);
long double log2l(long double x);
long double exp2l(long double x);
double round(double);
double trunc(double);
#endif

double freq2midi(double freq, double a_freq);
double cents2interv(double cents);
double interv2cents(double interv);
double midi2freq(double midi, double a_freq);

int cents2frac(double cents, int den);
int diatonic(int p);
int equaltemp(int n, double m /*ratio*/, double scale[]);
int equaltempc(int n, double m /*cents*/, double scale[]);
int lineartemp(int n,double m /*ratio*/,double generator /*ratio*/,
		   int countdownwards,
		   double scale[]);
int lineartempc(int n,double m /*cents*/,double generator /*cents*/,
		   int countdownwards,
		   double scale[]);

int mod(int n, int m);
double f_mod(double f, double m);
int nearestdiat(double pitch);
/* compare pointed double values (for use in qsort()) */
int fcompare(const double *x, const double *y);

#endif
